from artiq.experiment import *

class Set_All_Urukul_Freqs(EnvExperiment):
    """Urukul frecuencies setter

    Set the frecuencies/amplitudes of every Urukul channel
    """
    def build(self):
        # sets core device drivers as attributes
        self.setattr_device("core")

        for ch in range(4):
            # sets urukul0, channel 0-3 device drivers as attributes
            self.setattr_device(f"urukul0_ch{ch}")
            
            ### This two attributes will be shown in the GUI grouped by channel
            # use/don't use each channel
            self.setattr_argument(f"state_ch{ch}", BooleanValue(ch==0), f"canal_{ch}")
            # each channel's frequency
            self.setattr_argument(f"freq_ch{ch}", 
                                NumberValue(200.0*MHz, unit='MHz', scale=MHz, min=1*MHz, max=400*MHz),
                                f"canal_{ch}")

        self.all_channels = [self.urukul0_ch0, self.urukul0_ch1, self.urukul0_ch2, self.urukul0_ch3]
        self.states = [self.state_ch0, self.state_ch1, self.state_ch2, self.state_ch3]
        self.all_freqs = [self.freq_ch0, self.freq_ch1, self.freq_ch2, self.freq_ch3]

        self.use_freqs = []
        self.use_channels = []

        for state, ch_n, freq_n in zip(self.states, self.all_channels, self.all_freqs):
            if state:
                self.use_channels.append(ch_n)
                self.use_freqs.append(freq_n)


    @kernel
    def run(self):
        self.core.reset()
        # initialises CPLD all the selected channels
        for channel in self.use_channels:
            channel.cpld.init()
            channel.init()

        # defines global amplitude, for all channels
        amp = 1.0
        # defines attenuation variable from 0 (no attenuation) to max val (check) -31,5dB
        attenuation= 3.0*dB

        delay(10 * ms)

        # attenuation and frecuency to all channels
        for i in range(len(self.use_channels)):
#zip(self.use_channels, self.use_freqs):
            self.use_channels[i].set_att(attenuation)
            # writes frequency and amplitude variables to urukul channel thus outputting function
            self.use_channels[i].set(self.use_freqs[i], amplitude = amp)

        # turn on every selected channel
        for i in range(4):
            if self.states[i] == True:
                self.all_channels[i].sw.on()
            else:
                self.all_channels[i].sw.off()

